<?php

namespace App\Http\Controllers\Admin\Raharja;

use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Http\Request;
use App\Helpers\Helper;
use App\Model\Raharja\KelompokDriver;
use Yajra\Datatables\Datatables;
use Auth;
use Illuminate\Support\Facades\Storage;
use Validator;
use Hashids;

//load modelmu
class KelompokDriverController extends Controller
{
    /**
     * Title untuk judul di web
     * route digunakan untuk tempat resource (file path) + routing (route/web) diusahain sama ya biar gak ngubah"
     */
    private $title      = 'Kelompok Driver';                 /**jangan lupa diganti*/
    private $route      = 'admin.kelompok-driver.';          //path awal foldernya ajah (misal folder di admin/dashboard) => 'admin.dashboard' | jangan lupa diganti
    private $header     = 'Kelompok Driver';
    private $sub_header = 'Kelompok Driver';
    private $url        = 'admin.raharja.kelompok_driver.';

    public function __construct()
    {
        DB::enableQueryLog();
    }


    public function index()
    {
        Helper::swal();

        $data = [
            //bawaan
            'title'      => $this->title,
            'route'      => $this->route,
            'header'     => $this->header,
            'sub_header' => $this->sub_header,
        ];

        return view($this->url . 'index', $data);
    }
    public function getData($udd_id = null)
    {
        $query = KelompokDriver::query();

        return Datatables::of($query)
            ->addIndexColumn()
            ->addColumn('aksi', function ($query) {

                /**cek role */
                $aksi = '';
                $aksi .= "<a href='" . route($this->route . 'update', ['data_id' => Hashids::encode($query->id)]) . "' class='btn btn-sm btn-primary btn-edit'>Edit</a>";
                $aksi .= "<a href='javascript:;' data-route='" . route($this->route . 'delete_action', ['data_id' => Hashids::encode($query->id)]) . "' class='btn btn-danger btn-sm btn-delete'>Delete</a>";
                return $aksi;
            })
            ->rawColumns(['aksi'])
            ->toJson();
    }
    public function create()
    {
        $data = [
            //bawaan
            'title'         => $this->title,
            'route'         => $this->route,
            'header'        => $this->header,
            'sub_header'    => $this->sub_header,

        ];

        Helper::swal();

        return view($this->url . 'create', $data);
    }

    public function create_action(Request $request)
    {
        //declare post untuk simpan semua request yang diinputkan oleh form name=""
        $post = $request->all();
        $file = $request->file('logo');

        /**digunakan untuk set rule validator */
        $rules = [
            'logo'      => 'required|file|max:1000',
            'nama'      => 'required',
            'website'   => 'required',
            'deskripsi' => 'required',
        ];
        /**digunakan untuk set message dari validatornya yang akan keluar gimna, :attribue itu udah langsung mendeteksi inputtan dari name="attribue" */
        $alert = [
            'file'  => 'File maksimal 1MB'
        ];
        $validator = Validator::make($post, $rules, $alert);

        if ($validator->passes()) {
            /**menggunakan transaction */
            DB::beginTransaction();

            $nama_file = time()."_".$file->getClientOriginalName();

            Storage::putFileAs('public/images/kelompok_driver/', $file, $nama_file );

            $query            = new KelompokDriver();
            $query->logo      = $nama_file;
            $query->nama      = $request->nama;
            $query->website   = $request->website;
            $query->deskripsi = $request->deskripsi;
            $query->save();

            if ($query) {
                DB::commit();
                $message = 'Berhasil';
                return redirect(route($this->route . 'index'))->with('success', Helper::parsing_alert($message));
            } else {
                DB::rollback();
                $message = 'Gagal';
                return redirect()->back()->with('error', Helper::parsing_alert($message));
            }
        }
        /**kenapa menggunakan back ? karena baliknya pasti ke halaman sebelumnya */
        $message = Helper::parsing_alert($validator->errors()->all());
        return redirect()->back()->with('error', $message)->withInput();
    }

    public function update($data_id)
    {

        Helper::swal();


        $data_id = Hashids::decode($data_id);
        if (!empty($data_id)) {

            $cek_data = KelompokDriver::where('id', $data_id[0])->first();

            if ($cek_data) {
                $data = [
                    //bawaan
                    'title'         => $this->title,
                    'route'         => $this->route,
                    'header'        => $this->header,
                    'sub_header'    => $this->sub_header,
                    'data'          => $cek_data
                ];
                return view($this->url . 'update', $data);
            }
            $message = 'Id tidak ditemukan atau sudah dihapus';
            return redirect()->back()->with('error', $message);
        }
        $message = 'Id tdak ditemukan';
        return redirect()->back()->with('error', $message);
    }

    public function update_action(Request $request)
    {
        $post = $request->all();
       /**digunakan untuk set rule validator */
        $rules = [
            'logo'      => 'file|max:1000',
            'nama'      => 'required',
            'website'   => 'required',
            'deskripsi' => 'required',
        ];
        /**digunakan untuk set message dari validatornya yang akan keluar gimna, :attribue itu udah langsung mendeteksi inputtan dari name="attribue" */
        $alert = [];
        $validator = Validator::make($post, $rules, $alert);

        $data_id = Hashids::decode($request['id']);

        if (!empty($data_id)) {
            /**cek apakah data_idnya ada ? */
            $query = KelompokDriver::find($data_id[0]);

            if ($query) {
                /**cek apakah id data benar" ada di DB ? */
                if ($validator->passes()) {
                    /**validatornya */
                    DB::beginTransaction();
                    $file = $request->file('logo');
                    if($file){
                        $nama_file = time()."_".$file->getClientOriginalName();
                        Storage::putFileAs('public/images/kelompok_driver/', $file, $nama_file );
                        Storage::delete('public/images/kelompok_driver/'.$query->logo);
                        $query->logo = $nama_file;
                    }

                    $query->nama      = $request->nama;
                    $query->website   = $request->website;
                    $query->deskripsi = $request->deskripsi;
                    $query->save();

                    if ($query) {
                        DB::commit();
                        $message = 'Berhasil';
                        return redirect(route($this->route . 'index'))->with('success', Helper::parsing_alert($message));
                    } else {
                        DB::rollback();
                        $message = 'Gagal';
                        return redirect()->back()->with('error', Helper::parsing_alert($message));
                    }
                }

                $message = Helper::parsing_alert($validator->errors()->all());

                return redirect()->back()->with('error', $message);
            } else {
                $message = 'Id tdak ditemukan';
                return redirect()->back()->with('error', $message);
            }
        } else {
            $message = 'Id tidak boleh kosong';
            return redirect()->back()->with('error', $message);
        }
    }

    public function delete_action($id_modelmu)
    {
        $id = Hashids::decode($id_modelmu);
        /**cek idnya apakah ada ? */

        if (!empty($id)) {

            $cek_data = KelompokDriver::find($id[0]);

            if ($cek_data) {
                DB::beginTransaction();
                try {
                    Storage::delete('public/images/kelompok_driver/'.$cek_data->logo);
                    $delete = $cek_data->delete();
                    DB::commit();
                    $message = 'Sukses';
                    $response = [
                        'message' => $message,
                        'status'   => true,
                    ];
                    return response()->json($response);
                } catch (Exception $e) {
                    DB::rollback();
                    $message = 'Gagal';
                    $response = [
                        'message' => $message,
                        'status'   => false,
                    ];
                    return response()->json($response);
                }
            }
            $message = 'Id tidak ditemukan atau sudah dihapus';
            $response = [
                'message' => $message,
                'status'   => false,
            ];
            return response()->json($response);
        }
        $message = 'Id tdak ditemukan';
        $response = [
            'message' => $message,
            'status'   => false,
        ];
        return response()->json($response);
    }

}
