<?php

namespace App\Http\Controllers\Admin\Raharja;

use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Http\Request;
use App\Helpers\Helper;
use App\Model\Points;
use Yajra\Datatables\Datatables;
use Auth;
use Illuminate\Support\Facades\Storage;
use Validator;
use Hashids;

//load modelmu
class MasterPointsController extends Controller
{
    /**
     * Title untuk judul di web
     * route digunakan untuk tempat resource (file path) + routing (route/web) diusahain sama ya biar gak ngubah"
     */
    private $title      = 'Master Points';                 /**jangan lupa diganti*/
    private $route      = 'admin.master-points.';          //path awal foldernya ajah (misal folder di admin/dashboard) => 'admin.dashboard' | jangan lupa diganti
    private $header     = 'Points';
    private $sub_header = 'Points';
    private $url        = 'admin.raharja.master_points.';

     public function __construct()
    {
        DB::enableQueryLog();
        /** nyalakan jika sudah set rolenya, jika ini dinyalakan halaman ini tidak akan keluar */
        // $this->middleware('permission:searching-list|searching-create|searching-update|searching-delete', ['only' => ['index', 'create', 'update']]);
        // $this->middleware('permission:searching-create', ['only' => ['create', 'create_action']]);
        // $this->middleware('permission:searching-update', ['only' => ['update', 'update_action']]);
        // $this->middleware('permission:searching-delete', ['only' => ['delete']]);
    }

     /**
    * Ini contoh crud yang sudah jalan
    * index digunakna untuk tampilan awal dari menu yang akan dibuat
    * Untuk CRUD Biar cepat
    * Silahkan ganti
    * EloquentMu:: => dengan model anda
    * $id_modelmu => ganti dengan id di model anda
    * id_di_routenya => jangan lupa disesuaikan dengan yang dideclare di route
    */


    public function index()
    {
        Helper::swal();

        $data = [
            //bawaan
            'title'      => $this->title,
            'route'      => $this->route,
            'header'     => $this->header,
            'sub_header' => $this->sub_header,
        ];
        // dd($data);
        return view($this->url . 'index', $data);
    }
    public function getData($udd_id = null)
    {
        $query = Points::query();

        return Datatables::of($query)
            ->addIndexColumn()
            ->addColumn('aksi', function ($query) {

                /**cek role */
                $aksi = '';
                // if (Auth::user()->can('master-jenis-kantong-update')) {
                    $aksi .= "<a href='" . route($this->route . 'update', ['news_id' => Hashids::encode($query->id)]) . "' class='btn btn-sm btn-primary btn-edit'>Edit</a>";
                // }

                // if (Auth::user()->can('master-jenis-kantong-delete')) {
                    $aksi .= "<a href='javascript:;' data-route='" . route($this->route . 'delete_action', ['news_id' => Hashids::encode($query->id)]) . "' class='btn btn-danger btn-sm btn-delete'>Delete</a>";
                // }
                return $aksi;
            })
            ->rawColumns(['aksi'])
            ->toJson();
    }
    public function create()
    {
        $data = [
            //bawaan
            'title'         => $this->title,
            'route'         => $this->route,
            'header'        => $this->header,
            'sub_header'    => $this->sub_header,

        ];

        Helper::swal();

        return view($this->url . 'create', $data);
    }

    public function create_action(Request $request)
    {
        //declare post untuk simpan semua request yang diinputkan oleh form name=""
        $post = $request->all();
        //dd($post);
        /**digunakan untuk set rule validator */
        $rules = [
            'type'   => 'required',
            'value' => 'required',
        ];
        /**digunakan untuk set message dari validatornya yang akan keluar gimna, :attribue itu udah langsung mendeteksi inputtan dari name="attribue" */

        $validator = Validator::make($post, $rules);

        if ($validator->passes()) {
            /**menggunakan transaction */
            DB::beginTransaction();

            $query            = new Points();
            $query->type     = $request->type;
            $query->value   = $request->value;
            $query->save();

            if ($query) {
                DB::commit();
                $message = 'Berhasil';
                return redirect(route($this->route . 'index'))->with('success', Helper::parsing_alert($message));
            } else {
                DB::rollback();
                $message = 'Gagal';
                return redirect()->back()->with('error', Helper::parsing_alert($message));
            }
        }
        /**kenapa menggunakan back ? karena baliknya pasti ke halaman sebelumnya */
        $message = Helper::parsing_alert($validator->errors()->all());
        return redirect()->back()->with('error', $message)->withInput();
    }

    public function update($data_id)
    {

        Helper::swal();


        $data_id = Hashids::decode($data_id);
        if (!empty($data_id)) {

            $cek_data = Points::where('id', $data_id[0])->first();

            if ($cek_data) {
                $data = [
                    //bawaan
                    'title'         => $this->title,
                    'route'         => $this->route,
                    'header'        => $this->header,
                    'sub_header'    => $this->sub_header,
                    'data'          => $cek_data
                ];
                return view($this->url . 'update', $data);
            }
            $message = 'Id tidak ditemukan atau sudah dihapus';
            return redirect()->back()->with('error', $message);
        }
        $message = 'Id tdak ditemukan';
        return redirect()->back()->with('error', $message);
    }

    public function update_action(Request $request)
    {
        $post = $request->all();
       /**digunakan untuk set rule validator */
        $rules = [
            'type'   => 'required',
            'value' => 'required',
        ];
        /**digunakan untuk set message dari validatornya yang akan keluar gimna, :attribue itu udah langsung mendeteksi inputtan dari name="attribue" */
        $alert = [];
        $validator = Validator::make($post, $rules, $alert);

        $data_id = Hashids::decode($request['id']);

        if (!empty($data_id)) {
            /**cek apakah data_idnya ada ? */
            $query = Points::find($data_id[0]);

            if ($query) {
                /**cek apakah id data benar" ada di DB ? */
                if ($validator->passes()) {
                    /**validatornya */
                    DB::beginTransaction();

                    $query->type   = $request->type;
                    $query->value = $request->value;
                    $query->save();

                    if ($query) {
                        DB::commit();
                        $message = 'Berhasil';
                        return redirect(route($this->route . 'index'))->with('success', Helper::parsing_alert($message));
                    } else {
                        DB::rollback();
                        $message = 'Gagal';
                        return redirect()->back()->with('error', Helper::parsing_alert($message));
                    }
                }

                $message = Helper::parsing_alert($validator->errors()->all());

                return redirect()->back()->with('error', $message);
            } else {
                $message = 'Id tdak ditemukan';
                return redirect()->back()->with('error', $message);
            }
        } else {
            $message = 'Id tidak boleh kosong';
            return redirect()->back()->with('error', $message);
        }
    }

    public function delete_action($id_modelmu)
    {
        $id = Hashids::decode($id_modelmu);
        /**cek idnya apakah ada ? */

        if (!empty($id)) {

            $cek_data = Points::find($id[0]);

            if ($cek_data) {
                DB::beginTransaction();
                try {
                    $cek_data->delete();
                    DB::commit();
                    $message = 'Sukses';
                    $response = [
                        'message' => $message,
                        'status'   => true,
                    ];
                    return response()->json($response);
                } catch (Exception $e) {
                    DB::rollback();
                    $message = 'Gagal';
                    $response = [
                        'message' => $message,
                        'status'   => false,
                    ];
                    return response()->json($response);
                }
            }
            $message = 'Id tidak ditemukan atau sudah dihapus';
            $response = [
                'message' => $message,
                'status'   => false,
            ];
            return response()->json($response);
        }
        $message = 'Id tdak ditemukan';
        $response = [
            'message' => $message,
            'status'   => false,
        ];
        return response()->json($response);
    }

}
